@@ -10,6 +10,9 @@ pub fn concatenate_owned_unchecked(arrays: &[ArrayRef]) -> Result<ArrayRef> {
             "concat requires input of at least one array".to_string(),
         ));
     }
+    if arrays.len() == 1 {
+        return Ok(arrays[0].clone());
+    }
     let mut arrays_ref = Vec::with_capacity(arrays.len());
     let mut lengths = Vec::with_capacity(arrays.len());
     let mut capacity = 0;
