@@ -8,10 +8,10 @@ pub mod list;
 pub mod rolling;
 pub mod set;
 pub mod sort_partition;
+#[cfg(feature = "performant")]
 pub mod sorted_join;
 #[cfg(feature = "strings")]
 pub mod string;
-pub mod take;
 pub mod take_agg;
 
 /// Internal state of [SlicesIterator]
