@@ -2,6 +2,7 @@
 use crate::prelude::*;
 use arrow::{array::*, bitmap::Bitmap};
 use polars_arrow::prelude::ValueSize;
+use std::iter::Map;
 use std::marker::PhantomData;
 use std::sync::Arc;
 
@@ -20,6 +21,7 @@ mod ndarray;
 mod bitwise;
 #[cfg(feature = "object")]
 mod drop;
+mod from;
 pub(crate) mod list;
 pub(crate) mod logical;
 #[cfg(feature = "object")]
@@ -44,9 +46,10 @@ use polars_arrow::prelude::*;
 #[cfg(feature = "dtype-categorical")]
 use crate::chunked_array::categorical::RevMapping;
 use crate::series::IsSorted;
-use crate::utils::CustomIterTools;
+use crate::utils::{first_non_null, last_non_null, CustomIterTools};
 use bitflags::bitflags;
 use std::mem;
+use std::slice::Iter;
 
 #[cfg(not(feature = "dtype-categorical"))]
 pub struct RevMapping {}
@@ -136,17 +139,19 @@ pub type ChunkIdIter<'a> = std::iter::Map<std::slice::Iter<'a, ArrayRef>, fn(&Ar
 /// When multiplying two `ChunkArray'`s with different chunk sizes they cannot utilize [SIMD](https://en.wikipedia.org/wiki/SIMD) for instance.
 ///
 /// If you want to have predictable performance
-/// (no unexpected re-allocation of memory), it is advised to call the [rechunk](chunked_array/chunkops/trait.ChunkOps.html) after
+/// (no unexpected re-allocation of memory), it is advised to call the [ChunkedArray::rechunk] after
 /// multiple append operations.
 ///
 /// See also [`ChunkedArray::extend`] for appends within a chunk.
-pub struct ChunkedArray<T> {
+pub struct ChunkedArray<T: PolarsDataType> {
     pub(crate) field: Arc<Field>,
     pub(crate) chunks: Vec<ArrayRef>,
     phantom: PhantomData<T>,
+    /// TODO! remove this. Should be in the DataType
     /// maps categorical u32 indexes to String values
     pub(crate) categorical_map: Option<Arc<RevMapping>>,
     pub(crate) bit_settings: Settings,
+    length: IdxSize,
 }
 
 bitflags! {
@@ -157,7 +162,7 @@ bitflags! {
     const FAST_EXPLODE_LIST = 0x03;
 }}
 
-impl<T> ChunkedArray<T> {
+impl<T: PolarsDataType> ChunkedArray<T> {
     pub(crate) fn is_sorted(&self) -> bool {
         self.bit_settings.contains(Settings::SORTED_ASC)
     }
@@ -204,26 +209,26 @@ impl<T> ChunkedArray<T> {
 
     /// Get the index of the first non null value in this ChunkedArray.
     pub fn first_non_null(&self) -> Option<usize> {
-        let mut offset = 0;
-        for validity in self.iter_validities() {
-            if let Some(validity) = validity {
-                for (idx, is_valid) in validity.iter().enumerate() {
-                    if is_valid {
-                        return Some(offset + idx);
-                    }
-                }
-                offset += validity.len()
-            } else {
-                return Some(offset);
-            }
+        if self.is_empty() {
+            None
+        } else {
+            first_non_null(self.iter_validities())
         }
-        None
+    }
+
+    /// Get the index of the last non null value in this ChunkedArray.
+    pub fn last_non_null(&self) -> Option<usize> {
+        last_non_null(self.iter_validities(), self.length as usize)
     }
 
     /// Get the buffer of bits representing null values
     #[inline]
-    pub fn iter_validities(&self) -> impl Iterator<Item = Option<&Bitmap>> + '_ {
-        self.chunks.iter().map(|arr| arr.validity())
+    #[allow(clippy::type_complexity)]
+    pub fn iter_validities(&self) -> Map<Iter<'_, ArrayRef>, fn(&ArrayRef) -> Option<&Bitmap>> {
+        fn to_validity(arr: &ArrayRef) -> Option<&Bitmap> {
+            arr.validity()
+        }
+        self.chunks.iter().map(to_validity)
     }
 
     #[inline]
@@ -297,10 +302,20 @@ impl<T> ChunkedArray<T> {
     }
 
     /// A reference to the chunks
+    #[inline]
     pub fn chunks(&self) -> &Vec<ArrayRef> {
         &self.chunks
     }
 
+    /// A mutable reference to the chunks
+    ///
+    /// # Safety
+    /// The caller must ensure to not change the `DataType` or `length` of any of the chunks.
+    #[inline]
+    pub unsafe fn chunks_mut(&mut self) -> &mut Vec<ArrayRef> {
+        &mut self.chunks
+    }
+
     /// Returns true if contains a single chunk and has no null values
     pub fn is_optimal_aligned(&self) -> bool {
         self.chunks.len() == 1 && !self.has_validity()
@@ -324,7 +339,9 @@ impl<T> ChunkedArray<T> {
     /// ```
     pub fn append_array(&mut self, other: ArrayRef) -> Result<()> {
         if self.field.data_type() == other.data_type() {
+            let length = other.len() as IdxSize;
             self.chunks.push(other);
+            self.length += length;
             Ok(())
         } else {
             Err(PolarsError::SchemaMisMatch(
@@ -346,7 +363,9 @@ impl<T> ChunkedArray<T> {
             phantom: PhantomData,
             categorical_map: self.categorical_map.clone(),
             bit_settings: self.bit_settings,
+            length: 0,
         };
+        out.compute_len();
         if !keep_sorted {
             out.set_sorted2(IsSorted::Not);
         }
@@ -415,7 +434,6 @@ impl<T> ChunkedArray<T> {
 impl<T> ChunkedArray<T>
 where
     T: PolarsDataType,
-    ChunkedArray<T>: ChunkOps,
 {
     /// Should be used to match the chunk_id of another ChunkedArray.
     /// # Panics
@@ -452,76 +470,6 @@ where
     }
 }
 
-impl<T> ChunkedArray<T>
-where
-    T: PolarsDataType,
-{
-    /// Create a new ChunkedArray from existing chunks.
-    pub fn from_chunks(name: &str, chunks: Vec<ArrayRef>) -> Self {
-        // prevent List<Null> if the inner list type is known.
-        let datatype = if matches!(T::get_dtype(), DataType::List(_)) {
-            if let Some(arr) = chunks.get(0) {
-                arr.data_type().into()
-            } else {
-                T::get_dtype()
-            }
-        } else {
-            T::get_dtype()
-        };
-        let field = Arc::new(Field::new(name, datatype));
-        ChunkedArray {
-            field,
-            chunks,
-            phantom: PhantomData,
-            categorical_map: None,
-            bit_settings: Default::default(),
-        }
-    }
-}
-
-// A hack to save compiler bloat for null arrays
-impl Int32Chunked {
-    pub(crate) fn new_null(name: &str, len: usize) -> Self {
-        let arr = arrow::array::new_null_array(ArrowDataType::Null, len);
-        let field = Arc::new(Field::new(name, DataType::Null));
-        let chunks = vec![arr as ArrayRef];
-        ChunkedArray {
-            field,
-            chunks,
-            phantom: PhantomData,
-            categorical_map: None,
-            bit_settings: Default::default(),
-        }
-    }
-}
-
-impl<T> ChunkedArray<T>
-where
-    T: PolarsNumericType,
-{
-    /// Create a new ChunkedArray by taking ownership of the Vec. This operation is zero copy.
-    pub fn from_vec(name: &str, v: Vec<T::Native>) -> Self {
-        let arr = to_array::<T>(v, None);
-        Self::from_chunks(name, vec![arr])
-    }
-
-    /// Nullify values in slice with an existing null bitmap
-    pub fn new_from_owned_with_null_bitmap(
-        name: &str,
-        values: Vec<T::Native>,
-        buffer: Option<Bitmap>,
-    ) -> Self {
-        let arr = to_array::<T>(values, buffer);
-        ChunkedArray {
-            field: Arc::new(Field::new(name, T::get_dtype())),
-            chunks: vec![arr],
-            phantom: PhantomData,
-            categorical_map: None,
-            ..Default::default()
-        }
-    }
-}
-
 pub(crate) trait AsSinglePtr {
     /// Rechunk and return a ptr to the start of the array
     fn as_single_ptr(&mut self) -> Result<usize> {
@@ -548,7 +496,7 @@ impl AsSinglePtr for BooleanChunked {}
 impl AsSinglePtr for ListChunked {}
 impl AsSinglePtr for Utf8Chunked {}
 #[cfg(feature = "object")]
-impl<T> AsSinglePtr for ObjectChunked<T> {}
+impl<T: PolarsObject> AsSinglePtr for ObjectChunked<T> {}
 
 impl<T> ChunkedArray<T>
 where
@@ -592,7 +540,7 @@ where
     }
 }
 
-impl<T> Clone for ChunkedArray<T> {
+impl<T: PolarsDataType> Clone for ChunkedArray<T> {
     fn clone(&self) -> Self {
         ChunkedArray {
             field: self.field.clone(),
@@ -600,11 +548,12 @@ impl<T> Clone for ChunkedArray<T> {
             phantom: PhantomData,
             categorical_map: self.categorical_map.clone(),
             bit_settings: self.bit_settings,
+            length: self.length,
         }
     }
 }
 
-impl<T> AsRef<ChunkedArray<T>> for ChunkedArray<T> {
+impl<T: PolarsDataType> AsRef<ChunkedArray<T>> for ChunkedArray<T> {
     fn as_ref(&self) -> &ChunkedArray<T> {
         self
     }
@@ -754,7 +703,6 @@ pub(crate) mod test {
 
     fn assert_slice_equal<T>(ca: &ChunkedArray<T>, eq: &[T::Native])
     where
-        ChunkedArray<T>: ChunkOps,
         T: PolarsNumericType,
     {
         assert_eq!(
@@ -827,16 +775,6 @@ pub(crate) mod test {
         assert_eq!(Vec::from(&s.reverse()), &[Some("c"), None, Some("a")]);
     }
 
-    #[test]
-    fn test_null_sized_chunks() {
-        let mut s = Float64Chunked::new("s", &Vec::<f64>::new());
-        s.append(&Float64Chunked::new("s2", &[1., 2., 3.]));
-        dbg!(&s);
-
-        let s = Float64Chunked::new("s", &Vec::<f64>::new());
-        dbg!(&s.into_iter().next());
-    }
-
     #[test]
     #[cfg(feature = "dtype-categorical")]
     fn test_iter_categorical() {
