@@ -2,6 +2,7 @@
 use crate::chunked_array::builder::get_list_builder;
 use crate::chunked_array::ChunkedArray;
 use crate::datatypes::BooleanChunked;
+use crate::series::IsSorted;
 use crate::{datatypes::PolarsNumericType, prelude::*, utils::CustomIterTools};
 use arrow::compute;
 use arrow::types::simd::Simd;
@@ -68,15 +69,49 @@ where
     }
 
     fn min(&self) -> Option<T::Native> {
-        self.downcast_iter()
-            .filter_map(compute::aggregate::min_primitive)
-            .fold_first_(|acc, v| if acc < v { acc } else { v })
+        match self.is_sorted2() {
+            IsSorted::Ascending => {
+                self.first_non_null().and_then(|idx| {
+                    // Safety:
+                    // first_non_null returns in bound index
+                    unsafe { self.get_unchecked(idx) }
+                })
+            }
+            IsSorted::Descending => {
+                self.last_non_null().and_then(|idx| {
+                    // Safety:
+                    // last returns in bound index
+                    unsafe { self.get_unchecked(idx) }
+                })
+            }
+            IsSorted::Not => self
+                .downcast_iter()
+                .filter_map(compute::aggregate::min_primitive)
+                .fold_first_(|acc, v| if acc < v { acc } else { v }),
+        }
     }
 
     fn max(&self) -> Option<T::Native> {
-        self.downcast_iter()
-            .filter_map(compute::aggregate::max_primitive)
-            .fold_first_(|acc, v| if acc > v { acc } else { v })
+        match self.is_sorted2() {
+            IsSorted::Ascending => {
+                self.last_non_null().and_then(|idx| {
+                    // Safety:
+                    // first_non_null returns in bound index
+                    unsafe { self.get_unchecked(idx) }
+                })
+            }
+            IsSorted::Descending => {
+                self.first_non_null().and_then(|idx| {
+                    // Safety:
+                    // last returns in bound index
+                    unsafe { self.get_unchecked(idx) }
+                })
+            }
+            IsSorted::Not => self
+                .downcast_iter()
+                .filter_map(compute::aggregate::max_primitive)
+                .fold_first_(|acc, v| if acc > v { acc } else { v }),
+        }
     }
 
     fn mean(&self) -> Option<f64> {
@@ -380,7 +415,7 @@ impl ChunkQuantile<f64> for Float64Chunked {
 impl ChunkQuantile<String> for Utf8Chunked {}
 impl ChunkQuantile<Series> for ListChunked {}
 #[cfg(feature = "object")]
-impl<T> ChunkQuantile<Series> for ObjectChunked<T> {}
+impl<T: PolarsObject> ChunkQuantile<Series> for ObjectChunked<T> {}
 impl ChunkQuantile<bool> for BooleanChunked {}
 
 impl<T> ChunkVar<f64> for ChunkedArray<T>
@@ -448,7 +483,7 @@ impl ChunkVar<f64> for Float64Chunked {
 impl ChunkVar<String> for Utf8Chunked {}
 impl ChunkVar<Series> for ListChunked {}
 #[cfg(feature = "object")]
-impl<T> ChunkVar<Series> for ObjectChunked<T> {}
+impl<T: PolarsObject> ChunkVar<Series> for ObjectChunked<T> {}
 impl ChunkVar<bool> for BooleanChunked {}
 
 /// Booleans are casted to 1 or 0.
@@ -599,7 +634,7 @@ impl VarAggSeries for ListChunked {
     }
 }
 #[cfg(feature = "object")]
-impl<T> VarAggSeries for ObjectChunked<T> {
+impl<T: PolarsObject> VarAggSeries for ObjectChunked<T> {
     fn var_as_series(&self) -> Series {
         unimplemented!()
     }
@@ -703,7 +738,7 @@ impl QuantileAggSeries for ListChunked {
     }
 }
 #[cfg(feature = "object")]
-impl<T> QuantileAggSeries for ObjectChunked<T> {
+impl<T: PolarsObject> QuantileAggSeries for ObjectChunked<T> {
     fn quantile_as_series(
         &self,
         _quantile: f64,
@@ -792,7 +827,7 @@ impl ChunkAggSeries for ListChunked {
 }
 
 #[cfg(feature = "object")]
-impl<T> ChunkAggSeries for ObjectChunked<T> {}
+impl<T: PolarsObject> ChunkAggSeries for ObjectChunked<T> {}
 
 impl<T> ArgAgg for ChunkedArray<T>
 where
@@ -817,7 +852,7 @@ impl ArgAgg for Utf8Chunked {}
 impl ArgAgg for ListChunked {}
 
 #[cfg(feature = "object")]
-impl<T> ArgAgg for ObjectChunked<T> {}
+impl<T: PolarsObject> ArgAgg for ObjectChunked<T> {}
 
 #[cfg(test)]
 mod test {
