@@ -2,6 +2,8 @@ pub use arrow;
 #[cfg(feature = "temporal")]
 pub use chrono;
 
+#[cfg(feature = "private")]
+pub use ahash;
 #[cfg(feature = "private")]
 pub use num;
 #[cfg(feature = "private")]
@@ -11,3 +13,5 @@ pub use rayon;
 #[cfg(feature = "private")]
 #[cfg(any(feature = "strings", feature = "temporal"))]
 pub use regex;
+
+pub use crate::vector_hasher::_boost_hash_combine;
