@@ -10,14 +10,7 @@ use polars_arrow::compute::cast::cast;
 impl Series {
     /// Returns a reference to the Arrow ArrayRef
     pub fn array_ref(&self, chunk_idx: usize) -> &ArrayRef {
-        match self.dtype() {
-            #[cfg(feature = "dtype-struct")]
-            DataType::Struct(_) => {
-                let ca = self.struct_().unwrap();
-                ca.arrow_array()
-            }
-            _ => &self.chunks()[chunk_idx] as &ArrayRef,
-        }
+        &self.chunks()[chunk_idx] as &ArrayRef
     }
 
     /// Convert a chunk in the Series to the correct Arrow type.
@@ -25,6 +18,29 @@ impl Series {
     /// 1 on 1 mapping for logical/ categoricals, etc.
     pub fn to_arrow(&self, chunk_idx: usize) -> ArrayRef {
         match self.dtype() {
+            // special list branch to
+            // make sure that we recursively apply all logical types.
+            DataType::List(inner) => {
+                let ca = self.list().unwrap();
+                let arr = ca.chunks[chunk_idx].clone();
+                let arr = arr.as_any().downcast_ref::<ListArray<i64>>().unwrap();
+
+                let s = unsafe {
+                    Series::from_chunks_and_dtype_unchecked("", vec![arr.values().clone()], inner)
+                };
+                let new_values = s.to_arrow(0);
+
+                let data_type = ListArray::<i64>::default_datatype(inner.to_arrow());
+                let arr = unsafe {
+                    ListArray::<i64>::new_unchecked(
+                        data_type,
+                        arr.offsets().clone(),
+                        new_values,
+                        arr.validity().cloned(),
+                    )
+                };
+                Box::new(arr)
+            }
             #[cfg(feature = "dtype-categorical")]
             DataType::Categorical(_) => {
                 let ca = self.categorical().unwrap();
@@ -55,8 +71,6 @@ impl Series {
             }
             #[cfg(feature = "dtype-time")]
             DataType::Time => cast(&*self.chunks()[chunk_idx], &DataType::Time.to_arrow()).unwrap(),
-            #[cfg(feature = "dtype-struct")]
-            DataType::Struct(_) => self.array_ref(chunk_idx).clone(),
             _ => self.array_ref(chunk_idx).clone(),
         }
     }
