@@ -29,15 +29,26 @@ pub(crate) fn arrow_schema_to_empty_df(schema: &ArrowSchema) -> DataFrame {
     DataFrame::new_no_checks(columns)
 }
 
-#[cfg(any(feature = "ipc", feature = "parquet", feature = "json",))]
+#[cfg(any(
+    feature = "ipc",
+    feature = "parquet",
+    feature = "json",
+    feature = "ipc_streaming"
+))]
 pub(crate) fn apply_predicate(
     df: &mut DataFrame,
     predicate: Option<&dyn PhysicalIoExpr>,
+    parallel: bool,
 ) -> Result<()> {
     if let (Some(predicate), false) = (&predicate, df.is_empty()) {
         let s = predicate.evaluate(df)?;
         let mask = s.bool().expect("filter predicates was not of type boolean");
-        *df = df.filter(mask)?;
+
+        if parallel {
+            *df = df.filter(mask)?;
+        } else {
+            *df = df._filter_seq(mask)?;
+        }
     }
     Ok(())
 }
