@@ -46,7 +46,6 @@ use crate::dsl::function_expr::FunctionExpr;
 #[cfg(feature = "trigonometry")]
 use crate::dsl::function_expr::TrigonometricFunction;
 
-use polars_arrow::array::default_arrays::FromData;
 #[cfg(feature = "diff")]
 use polars_core::series::ops::NullBehavior;
 use polars_core::series::IsSorted;
@@ -306,12 +305,12 @@ impl Expr {
             |s| match s.dtype() {
                 DataType::Float32 => {
                     let ca = s.f32()?;
-                    let mask = ca.is_not_nan().fill_null(FillNullStrategy::One)?;
+                    let mask = ca.is_not_nan();
                     ca.filter(&mask).map(|ca| ca.into_series())
                 }
                 DataType::Float64 => {
                     let ca = s.f64()?;
-                    let mask = ca.is_not_nan().fill_null(FillNullStrategy::One)?;
+                    let mask = ca.is_not_nan();
                     ca.filter(&mask).map(|ca| ca.into_series())
                 }
                 _ => Ok(s),
@@ -859,47 +858,13 @@ impl Expr {
         }
     }
 
-    pub fn shift_and_fill_impl(self, periods: i64, fill_value: Expr) -> Self {
-        // Note:
-        // The order of the then | otherwise is important
-        if periods > 0 {
-            when(self.clone().apply(
-                move |s: Series| {
-                    let len = s.len();
-                    let mut bits = MutableBitmap::with_capacity(s.len());
-                    bits.extend_constant(periods as usize, false);
-                    bits.extend_constant(len.saturating_sub(periods as usize), true);
-                    let mask = BooleanArray::from_data_default(bits.into(), None);
-                    let ca: BooleanChunked = mask.into();
-                    Ok(ca.into_series())
-                },
-                GetOutput::from_type(DataType::Boolean),
-            ))
-            .then(self.shift(periods))
-            .otherwise(fill_value)
-        } else {
-            when(self.clone().apply(
-                move |s: Series| {
-                    let length = s.len() as i64;
-                    // periods is negative, so subtraction.
-                    let tipping_point = std::cmp::max(length + periods, 0);
-                    let mut bits = MutableBitmap::with_capacity(s.len());
-                    bits.extend_constant(tipping_point as usize, true);
-                    bits.extend_constant(-periods as usize, false);
-                    let mask = BooleanArray::from_data_default(bits.into(), None);
-                    let ca: BooleanChunked = mask.into();
-                    Ok(ca.into_series())
-                },
-                GetOutput::from_type(DataType::Boolean),
-            ))
-            .then(self.shift(periods))
-            .otherwise(fill_value)
-        }
-    }
-
     /// Shift the values in the array by some period and fill the resulting empty values.
     pub fn shift_and_fill<E: Into<Expr>>(self, periods: i64, fill_value: E) -> Self {
-        self.shift_and_fill_impl(periods, fill_value.into())
+        self.apply_many_private(
+            FunctionExpr::ShiftAndFill { periods },
+            &[fill_value.into()],
+            "shift_and_fill",
+        )
     }
 
     /// Get an array with the cumulative sum computed at every element
@@ -1137,9 +1102,11 @@ impl Expr {
 
     /// Replace the floating point `NaN` values by a value.
     pub fn fill_nan<E: Into<Expr>>(self, fill_value: E) -> Self {
-        when(self.clone().is_nan())
-            .then(fill_value.into())
-            .otherwise(self)
+        // we take the not branch so that self is truthy value of `when -> then -> otherwise`
+        // and that ensure we keep the name of `self`
+        when(self.clone().is_not_nan())
+            .then(self)
+            .otherwise(fill_value.into())
     }
     /// Count the values of the Series
     /// or
@@ -1244,7 +1211,7 @@ impl Expr {
         }
     }
 
-    /// Compute the arcsine of the given expression
+    /// Compute the inverse sine of the given expression
     #[cfg(feature = "trigonometry")]
     pub fn arcsin(self) -> Self {
         Expr::Function {
@@ -1259,7 +1226,7 @@ impl Expr {
         }
     }
 
-    /// Compute the arccosine of the given expression
+    /// Compute the inverse cosine of the given expression
     #[cfg(feature = "trigonometry")]
     pub fn arccos(self) -> Self {
         Expr::Function {
@@ -1274,7 +1241,7 @@ impl Expr {
         }
     }
 
-    /// Compute the arctangent of the given expression
+    /// Compute the inverse tangent of the given expression
     #[cfg(feature = "trigonometry")]
     pub fn arctan(self) -> Self {
         Expr::Function {
@@ -1334,7 +1301,7 @@ impl Expr {
         }
     }
 
-    /// Compute the hyperbolic sine of the given expression
+    /// Compute the inverse hyperbolic sine of the given expression
     #[cfg(feature = "trigonometry")]
     pub fn arcsinh(self) -> Self {
         Expr::Function {
@@ -1349,7 +1316,7 @@ impl Expr {
         }
     }
 
-    /// Compute the hyperbolic cosine of the given expression
+    /// Compute the inverse hyperbolic cosine of the given expression
     #[cfg(feature = "trigonometry")]
     pub fn arccosh(self) -> Self {
         Expr::Function {
@@ -1364,7 +1331,7 @@ impl Expr {
         }
     }
 
-    /// Compute the hyperbolic tangent of the given expression
+    /// Compute the inverse hyperbolic tangent of the given expression
     #[cfg(feature = "trigonometry")]
     pub fn arctanh(self) -> Self {
         Expr::Function {
@@ -1379,6 +1346,21 @@ impl Expr {
         }
     }
 
+    /// Compute the sign of the given expression
+    #[cfg(feature = "sign")]
+    pub fn sign(self) -> Self {
+        Expr::Function {
+            input: vec![self],
+            function: FunctionExpr::Sign,
+            options: FunctionOptions {
+                collect_groups: ApplyOptions::ApplyFlat,
+                input_wildcard_expansion: false,
+                auto_explode: false,
+                fmt_str: "sign",
+            },
+        }
+    }
+
     /// Filter a single column
     /// Should be used in aggregation context. If you want to filter on a DataFrame level, use
     /// [LazyFrame::filter](LazyFrame::filter)
@@ -1748,6 +1730,17 @@ impl Expr {
         )
     }
 
+    /// Apply a rolling skew
+    #[cfg_attr(docsrs, doc(cfg(all(feature = "rolling_window", feature = "moment"))))]
+    #[cfg(feature = "rolling_window")]
+    #[cfg(feature = "moment")]
+    pub fn rolling_skew(self, window_size: usize, bias: bool) -> Expr {
+        self.apply_private(
+            FunctionExpr::RollingSkew { window_size, bias },
+            "rolling_skew",
+        )
+    }
+
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     #[cfg(feature = "rolling_window")]
     /// Apply a custom function over a rolling/ moving window of the array.
@@ -1772,7 +1765,7 @@ impl Expr {
     /// This has quite some dynamic dispatch, so prefer rolling_min, max, mean, sum over this.
     pub fn rolling_apply_float<F>(self, window_size: usize, f: F) -> Expr
     where
-        F: 'static + Fn(&Float64Chunked) -> Option<f64> + Send + Sync + Copy,
+        F: 'static + FnMut(&mut Float64Chunked) -> Option<f64> + Send + Sync + Copy,
     {
         self.apply(
             move |s| {
@@ -1843,6 +1836,15 @@ impl Expr {
 
     #[cfg(feature = "moment")]
     #[cfg_attr(docsrs, doc(cfg(feature = "moment")))]
+    /// Compute the sample skewness of a data set.
+    ///
+    /// For normally distributed data, the skewness should be about zero. For
+    /// uni-modal continuous distributions, a skewness value greater than zero means
+    /// that there is more weight in the right tail of the distribution. The
+    /// function `skewtest` can be used to determine if the skewness value
+    /// is close enough to zero, statistically speaking.
+    ///
+    /// see: https://github.com/scipy/scipy/blob/47bb6febaa10658c72962b9615d5d5aa2513fa3a/scipy/stats/stats.py#L1024
     pub fn skew(self, bias: bool) -> Expr {
         self.apply(
             move |s| s.skew(bias).map(|opt_v| Series::new(s.name(), &[opt_v])),
@@ -2110,10 +2112,10 @@ impl Expr {
     #[cfg_attr(docsrs, doc(cfg(feature = "dtype-struct")))]
     /// Count all unique values and create a struct mapping value to count
     /// Note that it is better to turn multithreaded off in the aggregation context
-    pub fn value_counts(self, multithreaded: bool) -> Self {
+    pub fn value_counts(self, multithreaded: bool, sorted: bool) -> Self {
         self.apply(
             move |s| {
-                s.value_counts(multithreaded)
+                s.value_counts(multithreaded, sorted)
                     .map(|df| df.into_struct(s.name()).into_series())
             },
             GetOutput::map_field(|fld| {
