@@ -139,6 +139,11 @@ pub enum ALogicalPlan {
         inputs: Vec<Node>,
         options: UnionOptions,
     },
+    ExtContext {
+        input: Node,
+        contexts: Vec<Node>,
+        schema: SchemaRef,
+    },
 }
 
 impl Default for ALogicalPlan {
@@ -221,6 +226,7 @@ impl ALogicalPlan {
                     None => input_schema,
                 };
             }
+            ExtContext { schema, .. } => schema,
         };
         Cow::Borrowed(schema)
     }
@@ -228,7 +234,11 @@ impl ALogicalPlan {
 
 impl ALogicalPlan {
     /// Takes the expressions of an LP node and the inputs of that node and reconstruct
-    pub fn with_exprs_and_input(&self, mut exprs: Vec<Node>, inputs: Vec<Node>) -> ALogicalPlan {
+    pub fn with_exprs_and_input(
+        &self,
+        mut exprs: Vec<Node>,
+        mut inputs: Vec<Node>,
+    ) -> ALogicalPlan {
         use ALogicalPlan::*;
 
         match self {
@@ -441,6 +451,11 @@ impl ALogicalPlan {
                 options: *options,
                 schema: schema.clone(),
             },
+            ExtContext { schema, .. } => ExtContext {
+                input: inputs.pop().unwrap(),
+                contexts: inputs,
+                schema: schema.clone(),
+            },
         }
     }
 
@@ -527,6 +542,7 @@ impl ALogicalPlan {
                     container.push(*node)
                 }
             }
+            ExtContext { .. } => {}
         }
     }
 
@@ -573,6 +589,14 @@ impl ALogicalPlan {
             HStack { input, .. } => *input,
             Distinct { input, .. } => *input,
             Udf { input, .. } => *input,
+            ExtContext {
+                input, contexts, ..
+            } => {
+                for n in contexts {
+                    container.push_node(*n)
+                }
+                *input
+            }
             #[cfg(feature = "parquet")]
             ParquetScan { .. } => return,
             #[cfg(feature = "ipc")]
@@ -775,10 +799,9 @@ impl<'a> ALogicalPlanBuilder<'a> {
 
         let right_names: PlHashSet<_> = right_on
             .iter()
-            .map(|e| match self.expr_arena.get(*e) {
-                AExpr::Alias(_, name) => name.clone(),
-                AExpr::Column(name) => name.clone(),
-                _ => panic!("could not determine join column names"),
+            .map(|e| {
+                aexpr_to_root_column_name(*e, self.expr_arena)
+                    .expect("could not determine join column names")
             })
             .collect();
 
