@@ -1,6 +1,7 @@
 use crate::dsl::function_expr::FunctionExpr;
 use polars_core::prelude::*;
 use polars_core::utils::get_supertype;
+use std::borrow::Cow;
 
 use crate::logical_plan::optimizer::stack_opt::OptimizationRule;
 use crate::logical_plan::Context;
@@ -95,8 +96,12 @@ fn get_input(lp_arena: &Arena<ALogicalPlan>, lp_node: Node) -> [Option<Node>; 2]
     inputs
 }
 
-fn get_schema(lp_arena: &Arena<ALogicalPlan>, lp_node: Node) -> Option<&SchemaRef> {
-    get_input(lp_arena, lp_node)[0].map(|input| lp_arena.get(input).schema(lp_arena))
+fn get_schema(lp_arena: &Arena<ALogicalPlan>, lp_node: Node) -> Cow<'_, SchemaRef> {
+    match get_input(lp_arena, lp_node) {
+        [Some(input), _] => lp_arena.get(input).schema(lp_arena),
+        // files don't have an input, so we must take their schema
+        [None, _] => Cow::Borrowed(lp_arena.get(lp_node).scan_schema()),
+    }
 }
 
 fn get_aexpr_and_type<'a>(
@@ -132,12 +137,16 @@ impl OptimizationRule for TypeCoercionRule {
                 falsy: falsy_node,
                 predicate,
             } => {
-                let input_schema = get_schema(lp_arena, lp_node)?;
+                let input_schema = get_schema(lp_arena, lp_node);
                 let (truthy, type_true) =
-                    get_aexpr_and_type(expr_arena, truthy_node, input_schema)?;
-                let (falsy, type_false) = get_aexpr_and_type(expr_arena, falsy_node, input_schema)?;
+                    get_aexpr_and_type(expr_arena, truthy_node, &input_schema)?;
+                let (falsy, type_false) =
+                    get_aexpr_and_type(expr_arena, falsy_node, &input_schema)?;
 
-                if type_true == type_false {
+                if type_true == type_false ||
+                    // data type unknown still has to be set
+                    matches!(&type_true, DataType::Unknown) || matches!(&type_false, DataType::Unknown)
+                {
                     None
                 } else {
                     let st = get_supertype(&type_true, &type_false).expect("supertype");
@@ -179,9 +188,10 @@ impl OptimizationRule for TypeCoercionRule {
                 op,
                 right: node_right,
             } => {
-                let input_schema = get_schema(lp_arena, lp_node)?;
-                let (left, type_left) = get_aexpr_and_type(expr_arena, node_left, input_schema)?;
-                let (right, type_right) = get_aexpr_and_type(expr_arena, node_right, input_schema)?;
+                let input_schema = get_schema(lp_arena, lp_node);
+                let (left, type_left) = get_aexpr_and_type(expr_arena, node_left, &input_schema)?;
+                let (right, type_right) =
+                    get_aexpr_and_type(expr_arena, node_right, &input_schema)?;
 
                 // don't coerce string with number comparisons. They must error
                 match (&type_left, &type_right, op) {
@@ -210,6 +220,9 @@ impl OptimizationRule for TypeCoercionRule {
                     (DataType::Time, DataType::Utf8, op) if op.is_comparison() => {
                         print_date_str_comparison_warning()
                     }
+                    // structs can be arbitrarily nested, leave the complexity to the caller for now.
+                    #[cfg(feature = "dtype-struct")]
+                    (DataType::Struct(_), DataType::Struct(_), _op) => return None,
                     _ => {}
                 }
 
@@ -287,7 +300,10 @@ impl OptimizationRule for TypeCoercionRule {
                     }
                 }
 
-                if type_left == type_right || compare_cat_to_string || datetime_arithmetic {
+                if type_left == type_right || compare_cat_to_string || datetime_arithmetic  ||
+                    // data type unknown still has to be set
+                    matches!(&type_left, DataType::Unknown) || matches!(&type_right, DataType::Unknown)
+                {
                     None
                 } else {
                     let st = get_supertype(&type_left, &type_right)
@@ -345,15 +361,15 @@ impl OptimizationRule for TypeCoercionRule {
                 ref input,
                 options,
             } => {
-                let input_schema = get_schema(lp_arena, lp_node)?;
+                let input_schema = get_schema(lp_arena, lp_node);
                 let other_node = input[1];
-                let (_, type_left) = get_aexpr_and_type(expr_arena, input[0], input_schema)?;
-                let (_, type_other) = get_aexpr_and_type(expr_arena, other_node, input_schema)?;
+                let (_, type_left) = get_aexpr_and_type(expr_arena, input[0], &input_schema)?;
+                let (_, type_other) = get_aexpr_and_type(expr_arena, other_node, &input_schema)?;
 
                 match (&type_left, type_other) {
-                    (DataType::Categorical(Some(rev_map)), DataType::Utf8)
-                        if rev_map.is_global() =>
-                    {
+                    // cast both local and global string cache
+                    // note that there might not yet be a rev
+                    (DataType::Categorical(_), DataType::Utf8) => {
                         let mut input = input.clone();
 
                         let casted_expr = AExpr::Cast {
@@ -383,11 +399,11 @@ impl OptimizationRule for TypeCoercionRule {
                 ref input,
                 options,
             } => {
-                let input_schema = get_schema(lp_arena, lp_node)?;
+                let input_schema = get_schema(lp_arena, lp_node);
                 let other_node = input[1];
-                let (left, type_left) = get_aexpr_and_type(expr_arena, input[0], input_schema)?;
+                let (left, type_left) = get_aexpr_and_type(expr_arena, input[0], &input_schema)?;
                 let (fill_value, type_fill_value) =
-                    get_aexpr_and_type(expr_arena, other_node, input_schema)?;
+                    get_aexpr_and_type(expr_arena, other_node, &input_schema)?;
                 let super_type = get_supertype(&type_left, &type_fill_value).ok()?;
                 let super_type =
                     modify_supertype(super_type, left, fill_value, &type_left, &type_fill_value);
@@ -397,6 +413,56 @@ impl OptimizationRule for TypeCoercionRule {
                     options,
                 })
             }
+            AExpr::Function {
+                // only for `DataType::Unknown` as it still has to be set.
+                function: FunctionExpr::ShiftAndFill { periods },
+                ref input,
+                options,
+            } => {
+                let input_schema = get_schema(lp_arena, lp_node);
+                let self_node = input[0];
+                let other_node = input[1];
+                let (left, type_self) = get_aexpr_and_type(expr_arena, self_node, &input_schema)?;
+                let (fill_value, type_other) =
+                    get_aexpr_and_type(expr_arena, other_node, &input_schema)?;
+
+                if type_self == type_other {
+                    return None;
+                }
+
+                let super_type = get_supertype(&type_self, &type_other).ok()?;
+                let super_type =
+                    modify_supertype(super_type, left, fill_value, &type_self, &type_other);
+
+                // only cast if the type is not already the super type.
+                // this can prevent an expensive flattening and subsequent aggregation
+                // in a groupby context. To be able to cast the groups need to be
+                // flattened
+                let new_node_self = if type_self != super_type {
+                    expr_arena.add(AExpr::Cast {
+                        expr: self_node,
+                        data_type: super_type.clone(),
+                        strict: false,
+                    })
+                } else {
+                    self_node
+                };
+                let new_node_other = if type_other != super_type {
+                    expr_arena.add(AExpr::Cast {
+                        expr: other_node,
+                        data_type: super_type,
+                        strict: false,
+                    })
+                } else {
+                    other_node
+                };
+
+                Some(AExpr::Function {
+                    function: FunctionExpr::ShiftAndFill { periods },
+                    input: vec![new_node_self, new_node_other],
+                    options,
+                })
+            }
 
             _ => None,
         }
@@ -411,6 +477,7 @@ mod test {
     use polars_core::prelude::*;
 
     #[test]
+    #[cfg(feature = "dtype-categorical")]
     fn test_categorical_utf8() {
         let mut rules: Vec<Box<dyn OptimizationRule>> = vec![Box::new(TypeCoercionRule {})];
         let schema = Schema::from(vec![Field::new("fruits", DataType::Categorical(None))]);
