@@ -9,6 +9,12 @@ pub(crate) struct DropDuplicatesExec {
 
 impl Executor for DropDuplicatesExec {
     fn execute(&mut self, state: &mut ExecutionState) -> Result<DataFrame> {
+        #[cfg(debug_assertions)]
+        {
+            if state.verbose() {
+                println!("run DropDuplicatesExec")
+            }
+        }
         let df = self.input.execute(state)?;
         let subset = self.options.subset.as_ref().map(|v| &***v);
         let keep = self.options.keep_strategy;
