@@ -9,6 +9,12 @@ pub(crate) struct ExplodeExec {
 
 impl Executor for ExplodeExec {
     fn execute(&mut self, state: &mut ExecutionState) -> Result<DataFrame> {
+        #[cfg(debug_assertions)]
+        {
+            if state.verbose() {
+                println!("run ExplodeExec")
+            }
+        }
         let df = self.input.execute(state)?;
         df.explode(&self.columns)
     }
