@@ -18,6 +18,12 @@ impl Executor for GroupByRollingExec {
 
     #[cfg(feature = "dynamic_groupby")]
     fn execute(&mut self, state: &mut ExecutionState) -> Result<DataFrame> {
+        #[cfg(debug_assertions)]
+        {
+            if state.verbose() {
+                println!("run GroupbyRollingExec")
+            }
+        }
         let mut df = self.input.execute(state)?;
         df.as_single_chunk_par();
         state.set_schema(self.input_schema.clone());
