@@ -10,6 +10,12 @@ pub struct MeltExec {
 
 impl Executor for MeltExec {
     fn execute(&mut self, state: &mut ExecutionState) -> Result<DataFrame> {
+        #[cfg(debug_assertions)]
+        {
+            if state.verbose() {
+                println!("run MeltExec")
+            }
+        }
         let df = self.input.execute(state)?;
         let args = std::mem::take(Arc::make_mut(&mut self.args));
         df.melt2(args)
