@@ -16,6 +16,12 @@ pub struct ProjectionExec {
 
 impl Executor for ProjectionExec {
     fn execute(&mut self, state: &mut ExecutionState) -> Result<DataFrame> {
+        #[cfg(debug_assertions)]
+        {
+            if state.verbose() {
+                println!("run ProjectionExec")
+            }
+        }
         let df = self.input.execute(state)?;
         state.set_schema(self.input_schema.clone());
 
