@@ -87,8 +87,8 @@ pub fn apply_operator(left: &Series, right: &Series, op: Operator) -> Result<Ser
 }
 
 impl PhysicalExpr for BinaryExpr {
-    fn as_expression(&self) -> &Expr {
-        &self.expr
+    fn as_expression(&self) -> Option<&Expr> {
+        Some(&self.expr)
     }
 
     fn evaluate(&self, df: &DataFrame, state: &ExecutionState) -> Result<Series> {
@@ -202,8 +202,8 @@ impl PhysicalExpr for BinaryExpr {
                 // this prevents a series Arc alloc and a vec alloc per iteration
                 let dummy = Series::try_from(("dummy", vec![arr_l.clone()])).unwrap();
                 // keep logical type info
-                let dummy = dummy.cast(l.dtype()).unwrap();
-                let mut us = UnstableSeries::new(&dummy);
+                let mut dummy = dummy.cast(l.dtype()).unwrap();
+                let mut us = UnstableSeries::new(&mut dummy);
 
                 // this is now a list
                 let r = ac_r.aggregated_arity_operation();
@@ -220,8 +220,8 @@ impl PhysicalExpr for BinaryExpr {
 
                                 // Safety:
                                 // we are in bounds
-                                let arr = unsafe { arr_l.slice_unchecked(idx, 1) };
-                                us.swap(arr);
+                                let mut arr = unsafe { arr_l.slice_unchecked(idx, 1) };
+                                us.swap(&mut arr);
 
                                 let l = us.as_ref();
 
@@ -257,8 +257,8 @@ impl PhysicalExpr for BinaryExpr {
                 // this prevents a series Arc alloc and a vec alloc per iteration
                 let dummy = Series::try_from(("dummy", vec![arr_r.clone()])).unwrap();
                 // keep logical type info
-                let dummy = dummy.cast(r.dtype()).unwrap();
-                let mut us = UnstableSeries::new(&dummy);
+                let mut dummy = dummy.cast(r.dtype()).unwrap();
+                let mut us = UnstableSeries::new(&mut dummy);
 
                 let mut ca: ListChunked = l
                     .amortized_iter()
@@ -270,8 +270,8 @@ impl PhysicalExpr for BinaryExpr {
                                 // TODO: optimize this? Its slow.
                                 // Safety:
                                 // we are in bounds
-                                let arr = unsafe { arr_r.slice_unchecked(idx, 1) };
-                                us.swap(arr);
+                                let mut arr = unsafe { arr_r.slice_unchecked(idx, 1) };
+                                us.swap(&mut arr);
                                 let r = us.as_ref();
 
                                 apply_operator(l, r, self.op)
@@ -390,6 +390,16 @@ impl PhysicalExpr for BinaryExpr {
     fn as_stats_evaluator(&self) -> Option<&dyn polars_io::predicates::StatsEvaluator> {
         Some(self)
     }
+
+    fn is_valid_aggregation(&self) -> bool {
+        // we don't want:
+        // col(a) == lit(1)
+
+        // we do want
+        // col(a).sum() == lit(1)
+        (!self.left.is_literal() && self.left.is_valid_aggregation())
+            | (!self.right.is_literal() && self.right.is_valid_aggregation())
+    }
 }
 
 #[cfg(feature = "parquet")]
