@@ -24,10 +24,9 @@ impl IsNullExpr {
 }
 
 impl PhysicalExpr for IsNullExpr {
-    fn as_expression(&self) -> &Expr {
-        &self.expr
+    fn as_expression(&self) -> Option<&Expr> {
+        Some(&self.expr)
     }
-
     fn evaluate(&self, df: &DataFrame, state: &ExecutionState) -> Result<Series> {
         let series = self.physical_expr.evaluate(df, state)?;
         Ok(series.is_null().into_series())
@@ -57,6 +56,9 @@ impl PhysicalExpr for IsNullExpr {
     fn as_partitioned_aggregator(&self) -> Option<&dyn PartitionedAggregation> {
         Some(self)
     }
+    fn is_valid_aggregation(&self) -> bool {
+        self.physical_expr.is_valid_aggregation()
+    }
 }
 
 #[cfg(feature = "parquet")]
