@@ -22,10 +22,9 @@ impl TakeExpr {
 }
 
 impl PhysicalExpr for TakeExpr {
-    fn as_expression(&self) -> &Expr {
-        &self.expr
+    fn as_expression(&self) -> Option<&Expr> {
+        Some(&self.expr)
     }
-
     fn evaluate(&self, df: &DataFrame, state: &ExecutionState) -> Result<Series> {
         let series = self.phys_expr.evaluate(df, state)?;
         self.finish(df, state, series)
@@ -189,4 +188,8 @@ impl PhysicalExpr for TakeExpr {
     fn to_field(&self, input_schema: &Schema) -> Result<Field> {
         self.phys_expr.to_field(input_schema)
     }
+
+    fn is_valid_aggregation(&self) -> bool {
+        true
+    }
 }
