@@ -1,4 +1,5 @@
 use super::*;
+use crate::dsl::AggExpr::List;
 use polars_arrow::prelude::QuantileInterpolOptions;
 use polars_core::frame::explode::MeltArgs;
 use polars_core::series::ops::NullBehavior;
@@ -1509,7 +1510,7 @@ fn test_when_then_schema() -> Result<()> {
             .then(Null {}.lit())
             .otherwise(col("A"))])
         .schema();
-    assert_ne!(schema.get_index(0).unwrap().1, &DataType::Null);
+    assert_ne!(schema?.get_index(0).unwrap().1, &DataType::Null);
 
     Ok(())
 }
