@@ -93,10 +93,24 @@ pub(crate) fn has_wildcard(current_expr: &Expr) -> bool {
     has_expr(current_expr, |e| matches!(e, Expr::Wildcard))
 }
 
+// this one is used so much that it has its own function, to reduce inlining
+pub(crate) fn has_regex(current_expr: &Expr) -> bool {
+    has_expr(current_expr, |e| match e {
+        Expr::Column(name) => name.starts_with('^') && name.ends_with('$'),
+        _ => false,
+    })
+}
+
 pub(crate) fn has_nth(current_expr: &Expr) -> bool {
     has_expr(current_expr, |e| matches!(e, Expr::Nth(_)))
 }
 
+pub(crate) fn has_null(current_expr: &Expr) -> bool {
+    has_expr(current_expr, |e| {
+        matches!(e, Expr::Literal(LiteralValue::Null))
+    })
+}
+
 /// output name of expr
 pub(crate) fn expr_output_name(expr: &Expr) -> Result<Arc<str>> {
     for e in expr {
