@@ -58,7 +58,13 @@ pub trait DatetimeMethods: AsDatetime {
     /// Extract weekday from underlying NaiveDateTime representation.
     /// Returns the weekday number where monday = 0 and sunday = 6
     fn weekday(&self) -> UInt32Chunked {
-        cast_and_apply(self.as_datetime(), temporal::weekday)
+        let ca = self.as_datetime();
+        let f = match ca.time_unit() {
+            TimeUnit::Nanoseconds => datetime_to_weekday_ns,
+            TimeUnit::Microseconds => datetime_to_weekday_us,
+            TimeUnit::Milliseconds => datetime_to_weekday_ms,
+        };
+        ca.apply_kernel_cast::<UInt32Type>(&f)
     }
 
     /// Returns the ISO week number starting from 1.
