@@ -1,8 +1,9 @@
-from typing import Any, Callable, Dict, Sequence, Type
+from __future__ import annotations
 
-import numpy as np
+from typing import Any, Callable, Sequence
 
 from polars.datatypes import (
+    DTYPE_TEMPORAL_UNITS,
     Boolean,
     Categorical,
     Date,
@@ -28,12 +29,18 @@ try:
     from polars.polars import PySeries
 
     _DOCUMENTING = False
-except ImportError:  # pragma: no cover
+except ImportError:
     _DOCUMENTING = True
 
+try:
+    import numpy as np
+
+    _NUMPY_AVAILABLE = True
+except ImportError:
+    _NUMPY_AVAILABLE = False
 
 if not _DOCUMENTING:
-    _POLARS_TYPE_TO_CONSTRUCTOR: Dict[PolarsDataType, Callable] = {
+    _POLARS_TYPE_TO_CONSTRUCTOR: dict[PolarsDataType, Callable] = {
         Float32: PySeries.new_opt_f32,
         Float64: PySeries.new_opt_f64,
         Int8: PySeries.new_opt_i8,
@@ -53,21 +60,21 @@ if not _DOCUMENTING:
         Object: PySeries.new_object,
         Categorical: PySeries.new_str,
     }
+    for tu in DTYPE_TEMPORAL_UNITS:
+        _POLARS_TYPE_TO_CONSTRUCTOR[Datetime(tu)] = PySeries.new_opt_i64
 
 
 def polars_type_to_constructor(
     dtype: PolarsDataType,
-) -> Callable[[str, Sequence[Any], bool], "PySeries"]:
-    """
-    Get the right PySeries constructor for the given Polars dtype.
-    """
+) -> Callable[[str, Sequence[Any], bool], PySeries]:
+    """Get the right PySeries constructor for the given Polars dtype."""
     try:
         return _POLARS_TYPE_TO_CONSTRUCTOR[dtype]
     except KeyError:  # pragma: no cover
-        raise ValueError(f"Cannot construct PySeries for type {dtype}.")
+        raise ValueError(f"Cannot construct PySeries for type {dtype}.") from None
 
 
-if not _DOCUMENTING:
+if _NUMPY_AVAILABLE and not _DOCUMENTING:
     _NUMPY_TYPE_TO_CONSTRUCTOR = {
         np.float32: PySeries.new_f32,
         np.float64: PySeries.new_f64,
@@ -84,14 +91,16 @@ if not _DOCUMENTING:
     }
 
 
-def numpy_type_to_constructor(dtype: Type[np.dtype]) -> Callable[..., "PySeries"]:
-    """
-    Get the right PySeries constructor for the given Polars dtype.
-    """
+def numpy_type_to_constructor(dtype: type[np.dtype]) -> Callable[..., PySeries]:
+    """Get the right PySeries constructor for the given Polars dtype."""
     try:
         return _NUMPY_TYPE_TO_CONSTRUCTOR[dtype]
     except KeyError:
         return PySeries.new_object
+    except NameError:  # pragma: no cover
+        raise ImportError(
+            f"'numpy' is required to convert numpy dtype {dtype}."
+        ) from None
 
 
 if not _DOCUMENTING:
@@ -103,10 +112,8 @@ if not _DOCUMENTING:
     }
 
 
-def py_type_to_constructor(dtype: Type[Any]) -> Callable[..., "PySeries"]:
-    """
-    Get the right PySeries constructor for the given Python dtype.
-    """
+def py_type_to_constructor(dtype: type[Any]) -> Callable[..., PySeries]:
+    """Get the right PySeries constructor for the given Python dtype."""
     try:
         return _PY_TYPE_TO_CONSTRUCTOR[dtype]
     except KeyError:
