@@ -37,9 +37,11 @@ __all__ = [
     "Series",
     "all",
     "arg_where",
+    "argsort_by",
     "col",
     "concat",
     "concat_list",
+    "date_range",
     "element",
     "expr_to_lit_or_expr",
     "format",
@@ -53,6 +55,7 @@ __all__ = [
     "wrap_expr",
     "wrap_ldf",
     "wrap_s",
+    "_deser_and_exec",
     "_is_local_file",
     "_prepare_file_arg",
     "_scan_ds",
