@@ -1,30 +1,26 @@
-import warnings
-from datetime import date, datetime, timedelta
-from itertools import zip_longest
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    Iterable,
-    List,
-    Optional,
-    Sequence,
-    Tuple,
-    Type,
-    Union,
-)
+from __future__ import annotations
 
-import numpy as np
+import sys
+import warnings
+from contextlib import suppress
+from datetime import date, datetime, time, timedelta
+from itertools import zip_longest
+from typing import TYPE_CHECKING, Any, Iterable, Sequence
 
 from polars import internals as pli
 from polars.datatypes import (
     Categorical,
-    DataType,
+    ColumnsType,
     Date,
     Datetime,
     Duration,
     Float32,
+    List,
+    PolarsDataType,
     Time,
+    dtype_to_arrow_type,
+    dtype_to_py_type,
+    is_polars_dtype,
     py_type_to_arrow_type,
     py_type_to_dtype,
 )
@@ -33,55 +29,51 @@ from polars.datatypes_constructor import (
     polars_type_to_constructor,
     py_type_to_constructor,
 )
+from polars.utils import threadpool_size
+
+if TYPE_CHECKING:
+    import pandas as pd
 
 try:
     from polars.polars import PyDataFrame, PySeries
 
     _DOCUMENTING = False
-except ImportError:  # pragma: no cover
+except ImportError:
     _DOCUMENTING = True
 
-if TYPE_CHECKING:  # pragma: no cover
-    import pandas as pd
+try:
+    import numpy as np
+
+    _NUMPY_AVAILABLE = True
+except ImportError:
+    _NUMPY_AVAILABLE = False
+
+try:
     import pyarrow as pa
 
     _PYARROW_AVAILABLE = True
+except ImportError:
+    _PYARROW_AVAILABLE = False
+
+if sys.version_info >= (3, 8):
+    from typing import Literal
 else:
-    try:
-        import pyarrow as pa
+    from typing_extensions import Literal
 
-        _PYARROW_AVAILABLE = True
-    except ImportError:  # pragma: no cover
-        _PYARROW_AVAILABLE = False
-
-ColumnsType = Union[
-    Union[List[str], Sequence[str]],  # ['x','y','z']
-    Dict[str, Type[DataType]],  # {'x':date,'y':str,'z':int}
-    Sequence[Tuple[str, Type[DataType]]],  # [('x',date),('y',str),('z',int)]
-]
 
 ################################
 # Series constructor interface #
 ################################
 
 
-def series_to_pyseries(
-    name: str,
-    values: "pli.Series",
-) -> "PySeries":
-    """
-    Construct a PySeries from a Polars Series.
-    """
+def series_to_pyseries(name: str, values: pli.Series) -> PySeries:
+    """Construct a PySeries from a Polars Series."""
     values.rename(name, in_place=True)
     return values.inner()
 
 
-def arrow_to_pyseries(
-    name: str, values: "pa.Array", rechunk: bool = True
-) -> "PySeries":
-    """
-    Construct a PySeries from an Arrow array.
-    """
+def arrow_to_pyseries(name: str, values: pa.Array, rechunk: bool = True) -> PySeries:
+    """Construct a PySeries from an Arrow array."""
     array = coerce_arrow(values)
     if hasattr(array, "num_chunks"):
         if array.num_chunks > 1:
@@ -101,10 +93,8 @@ def arrow_to_pyseries(
 
 def numpy_to_pyseries(
     name: str, values: np.ndarray, strict: bool = True, nan_to_null: bool = False
-) -> "PySeries":
-    """
-    Construct a PySeries from a numpy array.
-    """
+) -> PySeries:
+    """Construct a PySeries from a numpy array."""
     if not values.flags["C_CONTIGUOUS"]:
         values = np.array(values)
 
@@ -122,20 +112,24 @@ def numpy_to_pyseries(
         return PySeries.new_object(name, values, strict)
 
 
-def _get_first_non_none(values: Sequence[Optional[Any]]) -> Any:
+def _get_first_non_none(values: Sequence[Any | None]) -> Any:
     """
     Return the first value from a sequence that isn't None.
 
     If sequence doesn't contain non-None values, return None.
+
     """
-    return next((v for v in values if v is not None), None)
+    if values is not None:
+        return next((v for v in values if v is not None), None)
 
 
-def sequence_from_anyvalue_or_object(name: str, values: Sequence[Any]) -> "PySeries":
-    """
-    Last resort conversion. AnyValues are most flexible and if they fail we go for object types
+def sequence_from_anyvalue_or_object(name: str, values: Sequence[Any]) -> PySeries:
     """
+    Last resort conversion.
 
+    AnyValues are most flexible and if they fail we go for object types
+
+    """
     try:
         return PySeries.new_from_anyvalues(name, values)
     # raised if we cannot convert to Wrap<AnyValue>
@@ -146,41 +140,82 @@ def sequence_from_anyvalue_or_object(name: str, values: Sequence[Any]) -> "PySer
 def sequence_to_pyseries(
     name: str,
     values: Sequence[Any],
-    dtype: Optional[Type[DataType]] = None,
+    dtype: PolarsDataType | None = None,
     strict: bool = True,
-) -> "PySeries":
-    """
-    Construct a PySeries from a sequence.
-    """
-    # Empty sequence defaults to Float32 type
+) -> PySeries:
+    """Construct a PySeries from a sequence."""
+    python_dtype: type | None = None
+    nested_dtype: PolarsDataType | type | None = None
+    temporal_unit: str | None = None
+
+    # empty sequence defaults to Float32 type
     if not values and dtype is None:
         dtype = Float32
+    # lists defer to subsequent handling; identify nested type
+    elif dtype == List:
+        nested_dtype = getattr(dtype, "inner", None)
+        python_dtype = list
 
-    if dtype is not None:
+    # infer temporal type handling
+    py_temporal_types = {date, datetime, timedelta, time}
+    pl_temporal_types = {Date, Datetime, Duration, Time}
+
+    value = _get_first_non_none(values)
+    if value is not None:
+        # this branch is for dtypes set with python types.
+        # eg. 'datetime.date/datetime.datetime'
+        # and values that are integers
+        # if this holds we take the physical branch
+        # if the values are also python types we take the temporal branch
+        if dtype in py_temporal_types and isinstance(value, int):
+            dtype = py_type_to_dtype(dtype)  # construct from integer
+        elif (
+            dtype in pl_temporal_types or type(dtype) in pl_temporal_types
+        ) and not isinstance(value, int):
+            temporal_unit = getattr(dtype, "tu", None)
+            python_dtype = dtype_to_py_type(dtype)  # type: ignore[arg-type]
+
+    # physical branch
+    # flat data
+    if (dtype is not None) and is_polars_dtype(dtype) and (python_dtype is None):
         constructor = polars_type_to_constructor(dtype)
         pyseries = constructor(name, values, strict)
 
         if dtype in (Date, Datetime, Duration, Time, Categorical):
             pyseries = pyseries.cast(dtype, True)
-
         return pyseries
 
     else:
-        value = _get_first_non_none(values)
-        dtype_ = type(value) if value is not None else float
+        if python_dtype is None:
+            python_dtype = float if (value is None) else type(value)
+
+        # temporal branch
+        if python_dtype in py_temporal_types:
+            if dtype is None:
+                dtype = py_type_to_dtype(python_dtype)  # construct from integer
+            elif dtype in py_temporal_types:
+                dtype = py_type_to_dtype(dtype)
+
+            # if no temporal unit given, we use anyvalues, so that we have one
+            # consistent level of entry that sets the units and timezones
+            # (e.g. we ignore them). They can be set afterwards.
+            if dtype == Datetime and temporal_unit is None:
+                return PySeries.new_from_anyvalues(name, values)
 
-        if dtype_ in {date, datetime, timedelta}:
             if not _PYARROW_AVAILABLE:  # pragma: no cover
                 raise ImportError(
-                    "'pyarrow' is required for converting a Sequence of date or datetime values to a PySeries."
+                    "'pyarrow' is required for converting a Sequence of date or"
+                    " datetime values to a PySeries."
                 )
             # let arrow infer dtype if not timedelta
             # arrow uses microsecond durations by default, not supported yet.
-            return arrow_to_pyseries(name, pa.array(values))
+            arrow_dtype = dtype_to_arrow_type(dtype)
+            return arrow_to_pyseries(name, pa.array(values, type=arrow_dtype))
 
-        elif dtype_ == list or dtype_ == tuple:
-            nested_value = _get_first_non_none(value)
-            nested_dtype = type(nested_value) if value is not None else float
+        elif python_dtype in (list, tuple):
+            if nested_dtype is None:
+                nested_value = _get_first_non_none(value)
+                nested_dtype = type(nested_value) if nested_value is not None else float
 
             # recursively call Series constructor
             if nested_dtype == list:
@@ -212,31 +247,36 @@ def sequence_to_pyseries(
                             count += 1
                     if equal_to_inner:
                         dtype = py_type_to_dtype(nested_dtype)
-                        try:
+                        with suppress(BaseException):
                             return PySeries.new_list(name, values, dtype)
-                        except BaseException:
-                            pass
                 # pass we create an object if we get here
             else:
                 try:
-                    nested_arrow_dtype = py_type_to_arrow_type(nested_dtype)
+                    to_arrow_type = (
+                        dtype_to_arrow_type
+                        if is_polars_dtype(nested_dtype)
+                        else py_type_to_arrow_type
+                    )
+                    nested_arrow_dtype = to_arrow_type(
+                        nested_dtype  # type: ignore[arg-type]
+                    )
                 except ValueError:  # pragma: no cover
                     return sequence_from_anyvalue_or_object(name, values)
                 try:
                     arrow_values = pa.array(values, pa.large_list(nested_arrow_dtype))
                     return arrow_to_pyseries(name, arrow_values)
-                except pa.lib.ArrowInvalid:
+                except (pa.lib.ArrowInvalid, pa.lib.ArrowTypeError):
                     pass
 
             # Convert mixed sequences like `[[12], "foo", 9]`
             return PySeries.new_object(name, values, strict)
 
-        elif dtype_ == pli.Series:
+        elif python_dtype == pli.Series:
             return PySeries.new_series_list(name, [v.inner() for v in values], strict)
-        elif dtype_ == PySeries:
+        elif python_dtype == PySeries:
             return PySeries.new_series_list(name, values, strict)
         else:
-            constructor = py_type_to_constructor(dtype_)
+            constructor = py_type_to_constructor(python_dtype)
 
             if constructor == PySeries.new_object:
                 try:
@@ -249,10 +289,10 @@ def sequence_to_pyseries(
 
 
 def _pandas_series_to_arrow(
-    values: Union["pd.Series", "pd.DatetimeIndex"],
+    values: pd.Series | pd.DatetimeIndex,
     nan_to_none: bool = True,
-    min_len: Optional[int] = None,
-) -> "pa.Array":
+    min_len: int | None = None,
+) -> pa.Array:
     """
     Convert a pandas Series to an Arrow Array.
 
@@ -263,31 +303,32 @@ def _pandas_series_to_arrow(
     nan_to_none
         Interpret `NaN` as missing values
     min_len
-        in case of null values, this length will be used to create a dummy f64 array (with all values set to null)
+        in case of null values, this length will be used to create a dummy f64 array
+        (with all values set to null)
 
     Returns
     -------
+    Arrow Array
+
     """
     dtype = values.dtype
     if dtype == "object" and len(values) > 0:
-        if isinstance(_get_first_non_none(values.values), str):  # type: ignore
-            return pa.array(values, pa.large_utf8(), from_pandas=nan_to_none)
+        first_non_none = _get_first_non_none(values.values)  # type: ignore[arg-type]
 
-        # array is null array, we set to a float64 array
-        if values.values[0] is None and min_len is not None:
-            return pa.nulls(min_len, pa.float64())
-        else:
-            return pa.array(values, from_pandas=nan_to_none)
+        if isinstance(first_non_none, str):
+            return pa.array(values, pa.large_utf8(), from_pandas=nan_to_none)
+        if first_non_none is None:
+            return pa.nulls(min_len, pa.large_utf8())
+
+        return pa.array(values, from_pandas=nan_to_none)
     else:
         return pa.array(values, from_pandas=nan_to_none)
 
 
 def pandas_to_pyseries(
-    name: str, values: Union["pd.Series", "pd.DatetimeIndex"], nan_to_none: bool = True
-) -> "PySeries":
-    """
-    Construct a PySeries from a pandas Series or DatetimeIndex.
-    """
+    name: str, values: pd.Series | pd.DatetimeIndex, nan_to_none: bool = True
+) -> PySeries:
+    """Construct a PySeries from a pandas Series or DatetimeIndex."""
     if not _PYARROW_AVAILABLE:  # pragma: no cover
         raise ImportError(
             "'pyarrow' is required when constructing a PySeries from a pandas Series."
@@ -306,12 +347,9 @@ def pandas_to_pyseries(
 
 
 def _handle_columns_arg(
-    data: List["PySeries"],
-    columns: Optional[Sequence[str]] = None,
-) -> List["PySeries"]:
-    """
-    Rename data according to columns argument.
-    """
+    data: list[PySeries], columns: Sequence[str] | None = None
+) -> list[PySeries]:
+    """Rename data according to columns argument."""
     if not columns:
         return data
     else:
@@ -325,13 +363,8 @@ def _handle_columns_arg(
             raise ValueError("Dimensions of columns arg must match data dimensions.")
 
 
-def _post_apply_columns(
-    pydf: "PyDataFrame",
-    columns: ColumnsType,
-) -> "PyDataFrame":
-    """
-    Apply 'columns' param _after_ PyDataFrame creation (if no alternative).
-    """
+def _post_apply_columns(pydf: PyDataFrame, columns: ColumnsType) -> PyDataFrame:
+    """Apply 'columns' param _after_ PyDataFrame creation (if no alternative)."""
     pydf_columns, pydf_dtypes = pydf.columns(), pydf.dtypes()
     columns, dtypes = _unpack_columns(columns or pydf_columns)
     if columns != pydf_columns:
@@ -348,18 +381,20 @@ def _post_apply_columns(
 
 
 def _unpack_columns(
-    columns: Optional[ColumnsType],
-    lookup_names: Optional[Iterable[str]] = None,
-    n_expected: Optional[int] = None,
-) -> Tuple[List[str], Dict[str, Type[DataType]]]:
+    columns: ColumnsType | None,
+    lookup_names: Iterable[str] | None = None,
+    n_expected: int | None = None,
+) -> tuple[list[str], dict[str, PolarsDataType]]:
     """
-    Unpack column names and create dtype lookup for any (name,dtype) pairs or schema dict input.
+    Unpack column names and create dtype lookup.
+
+    Works for any (name, dtype) pairs or schema dict input.
     """
     if isinstance(columns, dict):
         columns = list(columns.items())
     column_names = [
         (col or f"column_{i}") if isinstance(col, str) else col[0]
-        for i, col in enumerate((columns or []))
+        for i, col in enumerate(columns or [])
     ]
     if not column_names and n_expected:
         column_names = [f"column_{i}" for i in range(n_expected)]
@@ -377,12 +412,9 @@ def _unpack_columns(
 
 
 def dict_to_pydf(
-    data: Dict[str, Sequence[Any]],
-    columns: Optional[ColumnsType] = None,
-) -> "PyDataFrame":
-    """
-    Construct a PyDataFrame from a dictionary of sequences.
-    """
+    data: dict[str, Sequence[Any]], columns: ColumnsType | None = None
+) -> PyDataFrame:
+    """Construct a PyDataFrame from a dictionary of sequences."""
     if columns is not None:
         # the columns arg may also set the dtype of the series
         columns, dtypes = _unpack_columns(columns, lookup_names=data.keys())
@@ -398,18 +430,102 @@ def dict_to_pydf(
             ]
         data_series = _handle_columns_arg(data_series, columns=columns)
         return PyDataFrame(data_series)
+
+    if _NUMPY_AVAILABLE:
+        all_numpy = True
+        for val in data.values():
+            # only start a thread pool from a reasonable size.
+            all_numpy = all_numpy and isinstance(val, np.ndarray) and len(val) > 1000
+            if not all_numpy:
+                break
+
+        if all_numpy:
+            # yes, multi-threading was easier in python here
+            # we cannot run multiple threads that run python code
+            # and release the gil in pyo3
+            # it will deadlock.
+
+            # dummy is threaded
+            import multiprocessing.dummy
+
+            pool_size = threadpool_size()
+            pool = multiprocessing.dummy.Pool(pool_size)
+            data_series = pool.map(
+                lambda t: pli.Series(t[0], t[1]).inner(),
+                [(k, v) for k, v in data.items()],
+            )
+            return PyDataFrame(data_series)
+
     # fast path
     return PyDataFrame.read_dict(data)
 
 
+def sequence_to_pydf(
+    data: Sequence[Any],
+    columns: ColumnsType | None = None,
+    orient: Literal["col", "row"] | None = None,
+) -> PyDataFrame:
+    """Construct a PyDataFrame from a sequence."""
+    data_series: list[PySeries]
+
+    if len(data) == 0:
+        return dict_to_pydf({}, columns=columns)
+
+    elif isinstance(data[0], pli.Series):
+        series_names = [s.name for s in data]
+        columns, dtypes = _unpack_columns(columns or series_names, n_expected=len(data))
+        data_series = []
+        for i, s in enumerate(data):
+            if not s.name:  # TODO: Replace by `if s.name is None` once allowed
+                s.rename(columns[i], in_place=True)
+
+            new_dtype = dtypes.get(columns[i])
+            if new_dtype and new_dtype != s.dtype:
+                s = s.cast(new_dtype)
+
+            data_series.append(s.inner())
+
+    elif isinstance(data[0], dict):
+        pydf = PyDataFrame.read_dicts(data)
+        if columns:
+            pydf = _post_apply_columns(pydf, columns)
+        return pydf
+
+    elif isinstance(data[0], Sequence) and not isinstance(data[0], str):
+        # Infer orientation
+        if orient is None and columns is not None:
+            orient = "col" if len(columns) == len(data) else "row"
+
+        if orient == "row":
+            pydf = PyDataFrame.read_rows(data)
+            if columns:
+                pydf = _post_apply_columns(pydf, columns)
+            return pydf
+        elif orient == "col" or orient is None:
+            columns, dtypes = _unpack_columns(columns, n_expected=len(data))
+            data_series = [
+                pli.Series(columns[i], data[i], dtypes.get(columns[i])).inner()
+                for i in range(len(data))
+            ]
+        else:
+            raise ValueError(
+                f"orient must be one of {{'col', 'row', None}}, got {orient} instead."
+            )
+
+    else:
+        columns, dtypes = _unpack_columns(columns, n_expected=1)
+        data_series = [pli.Series(columns[0], data, dtypes.get(columns[0])).inner()]
+
+    data_series = _handle_columns_arg(data_series, columns=columns)
+    return PyDataFrame(data_series)
+
+
 def numpy_to_pydf(
     data: np.ndarray,
-    columns: Optional[ColumnsType] = None,
-    orient: Optional[str] = None,
-) -> "PyDataFrame":
-    """
-    Construct a PyDataFrame from a numpy ndarray.
-    """
+    columns: ColumnsType | None = None,
+    orient: Literal["col", "row"] | None = None,
+) -> PyDataFrame:
+    """Construct a PyDataFrame from a numpy ndarray."""
     shape = data.shape
     n_columns = (
         0
@@ -449,11 +565,15 @@ def numpy_to_pydf(
                 pli.Series(columns[i], data[:, i], dtypes.get(columns[i])).inner()
                 for i in range(n_columns)
             ]
-        else:
+        elif orient == "col":
             data_series = [
                 pli.Series(columns[i], data[i], dtypes.get(columns[i])).inner()
                 for i in range(n_columns)
             ]
+        else:
+            raise ValueError(
+                f"orient must be one of {{'col', 'row', None}}, got {orient} instead."
+            )
     else:
         raise ValueError("A numpy array should not have more than two dimensions.")
 
@@ -461,70 +581,10 @@ def numpy_to_pydf(
     return PyDataFrame(data_series)
 
 
-def sequence_to_pydf(
-    data: Sequence[Any],
-    columns: Optional[ColumnsType] = None,
-    orient: Optional[str] = None,
-) -> "PyDataFrame":
-    """
-    Construct a PyDataFrame from a sequence.
-    """
-    data_series: List["PySeries"]
-
-    if len(data) == 0:
-        return dict_to_pydf({}, columns=columns)
-
-    elif isinstance(data[0], pli.Series):
-        series_names = [s.name for s in data]
-        columns, dtypes = _unpack_columns(columns or series_names, n_expected=len(data))
-        data_series = []
-        for i, s in enumerate(data):
-            if not s.name:  # TODO: Replace by `if s.name is None` once allowed
-                s.rename(columns[i], in_place=True)
-
-            new_dtype = dtypes.get(columns[i])
-            if new_dtype and new_dtype != s.dtype:
-                s = s.cast(new_dtype)
-
-            data_series.append(s.inner())
-
-    elif isinstance(data[0], dict):
-        pydf = PyDataFrame.read_dicts(data)
-        if columns:
-            pydf = _post_apply_columns(pydf, columns)
-        return pydf
-
-    elif isinstance(data[0], Sequence) and not isinstance(data[0], str):
-        # Infer orientation
-        if orient is None and columns is not None:
-            orient = "col" if len(columns) == len(data) else "row"
-
-        if orient == "row":
-            pydf = PyDataFrame.read_rows(data)
-            if columns:
-                pydf = _post_apply_columns(pydf, columns)
-            return pydf
-        else:
-            columns, dtypes = _unpack_columns(columns, n_expected=len(data))
-            data_series = [
-                pli.Series(columns[i], data[i], dtypes.get(columns[i])).inner()
-                for i in range(len(data))
-            ]
-
-    else:
-        columns, dtypes = _unpack_columns(columns, n_expected=1)
-        data_series = [pli.Series(columns[0], data, dtypes.get(columns[0])).inner()]
-
-    data_series = _handle_columns_arg(data_series, columns=columns)
-    return PyDataFrame(data_series)
-
-
 def arrow_to_pydf(
-    data: "pa.Table", columns: Optional[ColumnsType] = None, rechunk: bool = True
-) -> "PyDataFrame":
-    """
-    Construct a PyDataFrame from an Arrow Table.
-    """
+    data: pa.Table, columns: ColumnsType | None = None, rechunk: bool = True
+) -> PyDataFrame:
+    """Construct a PyDataFrame from an Arrow Table."""
     if not _PYARROW_AVAILABLE:  # pragma: no cover
         raise ImportError(
             "'pyarrow' is required when constructing a PyDataFrame from an Arrow Table."
@@ -540,8 +600,8 @@ def arrow_to_pydf(
             ) from e
 
     data_dict = {}
-    # dictionaries cannot be build in different batches (categorical does not allow that)
-    # so we rechunk them and create them separate.
+    # dictionaries cannot be built in different batches (categorical does not allow
+    # that) so we rechunk them and create them separately.
     dictionary_cols = {}
     names = []
     for i, column in enumerate(data):
@@ -585,13 +645,8 @@ def arrow_to_pydf(
     return pydf
 
 
-def series_to_pydf(
-    data: "pli.Series",
-    columns: Optional[ColumnsType] = None,
-) -> "PyDataFrame":
-    """
-    Construct a PyDataFrame from a Polars Series.
-    """
+def series_to_pydf(data: pli.Series, columns: ColumnsType | None = None) -> PyDataFrame:
+    """Construct a PyDataFrame from a Polars Series."""
     data_series = [data.inner()]
     series_name = [s.name() for s in data_series]
     columns, dtypes = _unpack_columns(columns or series_name, n_expected=1)
@@ -605,17 +660,16 @@ def series_to_pydf(
 
 
 def pandas_to_pydf(
-    data: "pd.DataFrame",
-    columns: Optional[ColumnsType] = None,
+    data: pd.DataFrame,
+    columns: ColumnsType | None = None,
     rechunk: bool = True,
     nan_to_none: bool = True,
-) -> "PyDataFrame":
-    """
-    Construct a PyDataFrame from a pandas DataFrame.
-    """
+) -> PyDataFrame:
+    """Construct a PyDataFrame from a pandas DataFrame."""
     if not _PYARROW_AVAILABLE:  # pragma: no cover
         raise ImportError(
-            "'pyarrow' is required when constructing a PyDataFrame from a pandas DataFrame."
+            "'pyarrow' is required when constructing a PyDataFrame from a pandas"
+            " DataFrame."
         )
     length = data.shape[0]
     arrow_dict = {
@@ -628,7 +682,7 @@ def pandas_to_pydf(
     return arrow_to_pydf(arrow_table, columns=columns, rechunk=rechunk)
 
 
-def coerce_arrow(array: "pa.Array", rechunk: bool = True) -> "pa.Array":
+def coerce_arrow(array: pa.Array, rechunk: bool = True) -> pa.Array:
     # note: Decimal256 could not be cast to float
     if isinstance(array.type, pa.Decimal128Type):
         array = pa.compute.cast(array, pa.float64())
