@@ -167,9 +167,10 @@ impl PyLazyFrame {
             "utf8" => CsvEncoding::Utf8,
             "utf8-lossy" => CsvEncoding::LossyUtf8,
             e => {
-                return Err(
-                    PyPolarsErr::Other(format!("encoding not {} not implemented.", e)).into(),
-                )
+                return Err(PyValueError::new_err(format!(
+                    "encoding must be one of {{'utf8', 'utf8-lossy'}}, got {}",
+                    e
+                )))
             }
         };
 
@@ -452,6 +453,11 @@ impl PyLazyFrame {
         PyLazyGroupBy { lgb: Some(lazy_gb) }
     }
 
+    pub fn with_context(&self, contexts: Vec<PyLazyFrame>) -> PyLazyFrame {
+        let contexts = contexts.into_iter().map(|ldf| ldf.ldf).collect::<Vec<_>>();
+        self.ldf.clone().with_context(contexts).into()
+    }
+
     #[allow(clippy::too_many_arguments)]
     #[cfg(feature = "asof_join")]
     pub fn join_asof(
