@@ -195,8 +195,11 @@ impl PyExpr {
     pub fn count(&self) -> PyExpr {
         self.clone().inner.count().into()
     }
-    pub fn value_counts(&self, multithreaded: bool) -> PyExpr {
-        self.inner.clone().value_counts(multithreaded).into()
+    pub fn value_counts(&self, multithreaded: bool, sorted: bool) -> PyExpr {
+        self.inner
+            .clone()
+            .value_counts(multithreaded, sorted)
+            .into()
     }
     pub fn unique_counts(&self) -> PyExpr {
         self.inner.clone().unique_counts().into()
@@ -363,54 +366,71 @@ impl PyExpr {
         self.clone().inner.abs().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn sin(&self) -> PyExpr {
         self.clone().inner.sin().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn cos(&self) -> PyExpr {
         self.clone().inner.cos().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn tan(&self) -> PyExpr {
         self.clone().inner.tan().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn arcsin(&self) -> PyExpr {
         self.clone().inner.arcsin().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn arccos(&self) -> PyExpr {
         self.clone().inner.arccos().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn arctan(&self) -> PyExpr {
         self.clone().inner.arctan().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn sinh(&self) -> PyExpr {
         self.clone().inner.sinh().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn cosh(&self) -> PyExpr {
         self.clone().inner.cosh().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn tanh(&self) -> PyExpr {
         self.clone().inner.tanh().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn arcsinh(&self) -> PyExpr {
         self.clone().inner.arcsinh().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn arccosh(&self) -> PyExpr {
         self.clone().inner.arccosh().into()
     }
 
+    #[cfg(feature = "trigonometry")]
     pub fn arctanh(&self) -> PyExpr {
         self.clone().inner.arctanh().into()
     }
 
+    #[cfg(feature = "sign")]
+    pub fn sign(&self) -> PyExpr {
+        self.clone().inner.sign().into()
+    }
+
     pub fn is_duplicated(&self) -> PyExpr {
         self.clone().inner.is_duplicated().into()
     }
@@ -439,6 +459,7 @@ impl PyExpr {
         self.clone().inner.is_in(expr.inner).into()
     }
 
+    #[cfg(feature = "repeat_by")]
     pub fn repeat_by(&self, by: PyExpr) -> PyExpr {
         self.clone().inner.repeat_by(by.inner).into()
     }
@@ -745,6 +766,10 @@ impl PyExpr {
         self.inner.clone().arr().lengths().into()
     }
 
+    pub fn arr_contains(&self, other: PyExpr) -> PyExpr {
+        self.inner.clone().arr().contains(other.inner).into()
+    }
+
     pub fn year(&self) -> PyExpr {
         self.clone().inner.dt().year().into()
     }
@@ -887,7 +912,8 @@ impl PyExpr {
             let gil = Python::acquire_gil();
             let py = gil.python();
 
-            let out = call_lambda_with_series(py, s.clone(), &lambda, &pypolars);
+            let out = call_lambda_with_series(py, s.clone(), &lambda, &pypolars)
+                .expect("python function failed");
             match out.getattr(py, "_s") {
                 Ok(pyseries) => {
                     let pyseries = pyseries.extract::<PySeries>(py).unwrap();
@@ -1371,6 +1397,7 @@ impl PyExpr {
         self.inner.clone().diff(n, null_behavior).into()
     }
 
+    #[cfg(feature = "pct_change")]
     fn pct_change(&self, n: usize) -> Self {
         self.inner.clone().pct_change(n).into()
     }
