@@ -20,7 +20,7 @@ def test_fill_null() -> None:
     s = pl.Series("A", [dt, None])
 
     for fill_val in (dt, pl.lit(dt)):
-        out = s.fill_null(fill_val)  # type: ignore
+        out = s.fill_null(fill_val)  # type: ignore[arg-type]
 
         assert out.null_count() == 0
         assert out.dt[0] == dt
@@ -32,7 +32,7 @@ def test_fill_null() -> None:
     s = pl.Series("a", [dt1, dt2, dt3, None])
     dt_2 = date(2001, 1, 4)
     for fill_val in (dt_2, pl.lit(dt_2)):
-        out = s.fill_null(fill_val)  # type: ignore
+        out = s.fill_null(fill_val)  # type: ignore[arg-type]
 
         assert out.null_count() == 0
         assert out.dt[0] == dt1
@@ -104,20 +104,20 @@ def test_timestamp() -> None:
 
 
 def test_from_pydatetime() -> None:
-    dates = [
+    datetimes = [
         datetime(2021, 1, 1),
         datetime(2021, 1, 2),
         datetime(2021, 1, 3),
         datetime(2021, 1, 4, 12, 12),
         None,
     ]
-    s = pl.Series("name", dates)
+    s = pl.Series("name", datetimes)
     assert s.dtype == pl.Datetime
     assert s.name == "name"
     assert s.null_count() == 1
-    assert s.dt[0] == dates[0]
+    assert s.dt[0] == datetimes[0]
 
-    dates = [date(2021, 1, 1), date(2021, 1, 2), date(2021, 1, 3), None]  # type: ignore
+    dates = [date(2021, 1, 1), date(2021, 1, 2), date(2021, 1, 3), None]
     s = pl.Series("name", dates)
     assert s.dtype == pl.Date
     assert s.name == "name"
@@ -155,13 +155,13 @@ def test_datetime_consistency() -> None:
 def test_timezone() -> None:
     ts = pa.timestamp("s")
     data = pa.array([1000, 2000], type=ts)
-    s: pl.Series = pl.from_arrow(data)  # type: ignore
+    s: pl.Series = pl.from_arrow(data)  # type: ignore[assignment]
 
     # with timezone; we do expect a warning here
     tz_ts = pa.timestamp("s", tz="America/New_York")
     tz_data = pa.array([1000, 2000], type=tz_ts)
     # with pytest.warns(Warning):
-    tz_s: pl.Series = pl.from_arrow(tz_data)  # type: ignore
+    tz_s: pl.Series = pl.from_arrow(tz_data)  # type: ignore[assignment]
 
     # timezones have no effect, i.e. `s` equals `tz_s`
     assert not s.series_equal(tz_s)
@@ -291,8 +291,8 @@ def test_date_comp() -> None:
     assert (a < one).to_list() == [False, False]
     assert (a <= one).to_list() == [True, False]
 
-    one = date(2001, 1, 1)  # type: ignore
-    two = date(2001, 1, 2)  # type: ignore
+    one = date(2001, 1, 1)  # type: ignore[assignment]
+    two = date(2001, 1, 2)  # type: ignore[assignment]
     a = pl.Series("a", [one, two])
     assert (a == one).to_list() == [True, False]
     assert (a != one).to_list() == [False, True]
@@ -302,14 +302,14 @@ def test_date_comp() -> None:
     assert (a <= one).to_list() == [True, False]
 
     # also test if the conversion stays correct with wide date ranges
-    one = date(201, 1, 1)  # type: ignore
-    two = date(201, 1, 2)  # type: ignore
+    one = date(201, 1, 1)  # type: ignore[assignment]
+    two = date(201, 1, 2)  # type: ignore[assignment]
     a = pl.Series("a", [one, two])
     assert (a == one).to_list() == [True, False]
     assert (a == two).to_list() == [False, True]
 
-    one = date(5001, 1, 1)  # type: ignore
-    two = date(5001, 1, 2)  # type: ignore
+    one = date(5001, 1, 1)  # type: ignore[assignment]
+    two = date(5001, 1, 2)  # type: ignore[assignment]
     a = pl.Series("a", [one, two])
     assert (a == one).to_list() == [True, False]
     assert (a == two).to_list() == [False, True]
@@ -531,7 +531,7 @@ def test_microseconds_accuracy() -> None:
         ),
     )
 
-    assert pl.from_arrow(a)["timestamp"].to_list() == timestamps  # type: ignore
+    assert pl.from_arrow(a)["timestamp"].to_list() == timestamps
 
 
 def test_cast_time_units() -> None:
@@ -741,28 +741,26 @@ def test_groupby_rolling_mean_3020() -> None:
 
 def test_asof_join() -> None:
     fmt = "%F %T%.3f"
-    dates = """2016-05-25 13:30:00.023
-2016-05-25 13:30:00.023
-2016-05-25 13:30:00.030
-2016-05-25 13:30:00.041
-2016-05-25 13:30:00.048
-2016-05-25 13:30:00.049
-2016-05-25 13:30:00.072
-2016-05-25 13:30:00.075""".split(
-        "\n"
-    )
-
-    ticker = """GOOG
-MSFT
-MSFT
-MSFT
-GOOG
-AAPL
-GOOG
-MSFT""".split(
-        "\n"
-    )
-
+    dates = [
+        "2016-05-25 13:30:00.023",
+        "2016-05-25 13:30:00.023",
+        "2016-05-25 13:30:00.030",
+        "2016-05-25 13:30:00.041",
+        "2016-05-25 13:30:00.048",
+        "2016-05-25 13:30:00.049",
+        "2016-05-25 13:30:00.072",
+        "2016-05-25 13:30:00.075",
+    ]
+    ticker = [
+        "GOOG",
+        "MSFT",
+        "MSFT",
+        "MSFT",
+        "GOOG",
+        "AAPL",
+        "GOOG",
+        "MSFT",
+    ]
     quotes = pl.DataFrame(
         {
             "dates": pl.Series(dates).str.strptime(pl.Datetime, fmt=fmt),
@@ -771,22 +769,20 @@ MSFT""".split(
         }
     )
 
-    dates = """2016-05-25 13:30:00.023
-2016-05-25 13:30:00.038
-2016-05-25 13:30:00.048
-2016-05-25 13:30:00.048
-2016-05-25 13:30:00.048""".split(
-        "\n"
-    )
-
-    ticker = """MSFT
-MSFT
-GOOG
-GOOG
-AAPL""".split(
-        "\n"
-    )
-
+    dates = [
+        "2016-05-25 13:30:00.023",
+        "2016-05-25 13:30:00.038",
+        "2016-05-25 13:30:00.048",
+        "2016-05-25 13:30:00.048",
+        "2016-05-25 13:30:00.048",
+    ]
+    ticker = [
+        "MSFT",
+        "MSFT",
+        "GOOG",
+        "GOOG",
+        "AAPL",
+    ]
     trades = pl.DataFrame(
         {
             "dates": pl.Series(dates).str.strptime(pl.Datetime, fmt=fmt),
@@ -887,7 +883,8 @@ def test_datetime_strptime_patterns() -> None:
     df = pl.Series(
         "date",
         [
-            "09-05-2019" "2018-09-05",
+            "09-05-2019",
+            "2018-09-05",
             "2018-09-05T04:05:01",
             "2018-09-05T04:24:01.9",
             "2018-09-05T04:24:02.11",
@@ -1222,3 +1219,55 @@ def test_time_microseconds_3843() -> None:
 def test_year_empty_df() -> None:
     df = pl.DataFrame(pl.Series(name="date", dtype=pl.Date))
     assert df.select(pl.col("date").dt.year()).dtypes == [pl.Int32]
+
+
+def test_sum_duration() -> None:
+    assert pl.DataFrame(
+        [
+            {"name": "Jen", "duration": timedelta(seconds=60)},
+            {"name": "Mike", "duration": timedelta(seconds=30)},
+            {"name": "Jen", "duration": timedelta(seconds=60)},
+        ]
+    ).select(
+        [pl.col("duration").sum(), pl.col("duration").dt.seconds().alias("sec").sum()]
+    ).to_dict(
+        False
+    ) == {
+        "duration": [timedelta(seconds=150)],
+        "sec": [150],
+    }
+
+
+def test_supertype_timezones_4174() -> None:
+    df = pl.DataFrame(
+        {
+            "dt": pl.date_range(datetime(2020, 3, 1), datetime(2020, 5, 1), "1mo"),
+        }
+    ).with_columns(
+        [
+            pl.col("dt").dt.with_time_zone("Europe/London").suffix("_London"),
+        ]
+    )
+
+    # test if this runs without error
+    date_to_fill = df["dt_London"][0]
+    df["dt_London"] = df["dt_London"].shift_and_fill(1, date_to_fill)
+
+
+def test_weekday() -> None:
+    # monday
+    s = pl.Series([datetime(2020, 1, 6)])
+
+    for tu in ["ns", "us", "ms"]:
+        assert s.dt.cast_time_unit(tu).dt.weekday()[0] == 0
+
+    assert s.cast(pl.Date).dt.weekday()[0] == 0
+
+
+def test_from_dict_tu_consistency() -> None:
+    tz = pytz.timezone("PRC")
+    dt = datetime(2020, 8, 1, 12, 0, 0, tzinfo=tz)
+    from_dict = pl.from_dict({"dt": [dt]})
+    from_dicts = pl.from_dicts([{"dt": dt}])
+
+    assert from_dict.dtypes == from_dicts.dtypes
