@@ -53,10 +53,10 @@ def test_flatten_explode() -> None:
     df = pl.Series("a", ["Hello", "World"])
     expected = pl.Series("a", ["H", "e", "l", "l", "o", "W", "o", "r", "l", "d"])
 
-    result: pl.Series = df.to_frame().select(pl.col("a").flatten())[:, 0]  # type: ignore
+    result = df.to_frame().select(pl.col("a").flatten())[:, 0]
     assert_series_equal(result, expected)
 
-    result: pl.Series = df.to_frame().select(pl.col("a").explode())[:, 0]  # type: ignore
+    result = df.to_frame().select(pl.col("a").explode())[:, 0]
     assert_series_equal(result, expected)
 
 
@@ -125,7 +125,7 @@ def test_wildcard_expansion() -> None:
 
 
 def test_split_exact() -> None:
-    df = pl.DataFrame(dict(x=["a_a", None, "b", "c_c"]))
+    df = pl.DataFrame({"x": ["a_a", None, "b", "c_c"]})
     out = df.select([pl.col("x").str.split_exact("_", 2, inclusive=False)]).unnest("x")
 
     expected = pl.DataFrame(
@@ -294,3 +294,22 @@ def test_arr_contains() -> None:
     ).collect().to_dict(False) == {
         "str_list": [["cat", "mouse", "dog"], ["dog", "mouse", "cat"]]
     }
+
+
+def test_rank_so_4109() -> None:
+    df = pl.from_dict(
+        {
+            "id": [1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4],
+            "rank": [None, 3, 2, 4, 1, 4, 3, 2, 1, None, 3, 4, 4, 1, None, 3],
+        }
+    ).sort(by=["id", "rank"])
+
+    assert df.groupby("id").agg(pl.col("rank").rank()).to_dict(False) == {
+        "id": [1, 2, 3, 4],
+        "rank": [
+            [1.0, 2.0, 3.0, 4.0],
+            [1.0, 2.0, 3.0, 4.0],
+            [1.0, 2.0, 3.0, 4.0],
+            [1.0, 2.0, 3.0, 4.0],
+        ],
+    }
