@@ -116,14 +116,14 @@ def test_struct_function_expansion() -> None:
 def test_value_counts_expr() -> None:
     df = pl.DataFrame(
         {
-            "id": ["a", "b", "b", "c", "c", "c"],
+            "id": ["a", "b", "b", "c", "c", "c", "d", "d"],
         }
     )
 
     out = (
         df.select(
             [
-                pl.col("id").value_counts(),
+                pl.col("id").value_counts(sort=True),
             ]
         )
         .to_series()
@@ -132,6 +132,7 @@ def test_value_counts_expr() -> None:
     assert out == [
         {"id": "c", "counts": 3},
         {"id": "b", "counts": 2},
+        {"id": "d", "counts": 2},
         {"id": "a", "counts": 1},
     ]
 
@@ -179,7 +180,12 @@ def test_struct_cols() -> None:
     """Test that struct columns can be imported and work as expected."""
 
     def build_struct_df(data: list) -> DataFrame:
-        """Build Polars df from list of dicts. Can't import directly because of issue #3145."""
+        """
+        Build Polars df from list of dicts.
+
+        Can't import directly because of issue #3145.
+
+        """
         arrow_df = pa.Table.from_pylist(data)
         polars_df = pl.from_arrow(arrow_df)
         assert isinstance(polars_df, DataFrame)
@@ -478,12 +484,18 @@ def test_struct_schema_on_append_extend_3452() -> None:
     housing1, housing2 = pl.Series(housing1_data), pl.Series(housing2_data)
     with pytest.raises(
         pl.SchemaError,
-        match="cannot append field with name: address to struct with field name: city, please check your schema",
+        match=(
+            "cannot append field with name: address to struct with field name: city,"
+            " please check your schema"
+        ),
     ):
         housing1.append(housing2, append_chunks=True)
     with pytest.raises(
         pl.SchemaError,
-        match="cannot extend field with name: address to struct with field name: city, please check your schema",
+        match=(
+            "cannot extend field with name: address to struct with field name: city,"
+            " please check your schema"
+        ),
     ):
         housing1.append(housing2, append_chunks=False)
 
@@ -554,3 +566,51 @@ def test_nested_explode_4026() -> None:
         ],
         "day": ["monday", "monday"],
     }
+
+
+def test_nested_struct_sliced_append() -> None:
+    s = pl.Series(
+        [
+            {
+                "_experience": {
+                    "aaid": {
+                        "id": "A",
+                        "namespace": {"code": "alpha"},
+                    }
+                }
+            },
+            {
+                "_experience": {
+                    "aaid": {
+                        "id": "B",
+                        "namespace": {"code": "bravo"},
+                    },
+                }
+            },
+            {
+                "_experience": {
+                    "aaid": {
+                        "id": "D",
+                        "namespace": {"code": "delta"},
+                    }
+                }
+            },
+        ]
+    )
+    s2 = s[1:]
+    s.append(s2)
+
+    assert s.to_list() == [
+        {"_experience": {"aaid": {"id": "A", "namespace": {"code": "alpha"}}}},
+        {"_experience": {"aaid": {"id": "B", "namespace": {"code": "bravo"}}}},
+        {"_experience": {"aaid": {"id": "D", "namespace": {"code": "delta"}}}},
+        {"_experience": {"aaid": {"id": "B", "namespace": {"code": "bravo"}}}},
+        {"_experience": {"aaid": {"id": "D", "namespace": {"code": "delta"}}}},
+    ]
+
+
+def test_struct_groupby_field_agg_4216() -> None:
+    df = pl.DataFrame([{"a": {"b": 1}, "c": 0}])
+    assert df.groupby("c").agg(pl.col("a").struct.field("b").count()).to_dict(
+        False
+    ) == {"c": [0], "b": [1]}
